package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * 逻辑定值-简单模式定值数据寄存器地址
 *
 * @author <a href="mailto:scwsl@foxmail.com>王仕龙</a>
 * 2024/11/28 0:31
 */
@Getter
@Setter
@SuperBuilder
public class SimpleLogicMode extends Command {
    @Serial
    private static final long serialVersionUID = 8899565585137994751L;
    /**
     * 开井时间，以秒为单位的时间
     */
    private int wellOpenTimeTimestamp;
    /**
     * 开井套压
     */
    private int wellOpenPressureValue;
    /**
     * 开井油压
     */
    private int wellOpenOilPressureValue;
    /**
     * 开井载荷因子预设值
     */
    private int wellOpenLoadFactorPresets;
    /**
     * 关井时间
     */
    private int wellCloseTimeTimestamp;
    /**
     * 关井套压
     */
    private int wellClosePressureValue;
    /**
     * 关井油压
     */
    private int wellCloseOilPressureValue;
    /**
     * 关井流量
     */
    private int wellCloseFlowValue;
    /**
     * 最小开井时间
     */
    private int inWellOpenTimeDuration;
    /**
     * 最大开井时间
     */
    private int axWellOpenTimeDuration;
    /**
     * 最小关井时间
     */
    private int inWellCloseTimeDuration;
    /**
     * 最大关井时间
     */
    private int axWellCloseTimeDuration;
    /**
     * 压力稳定时间
     */
    private int ressureStabilizationDuration;
    /**
     * 流量稳定时间
     */
    private int lowStabilizationDuration;
    /**
     * 载荷因子稳定时间
     */
    private int loadFactorStabilizationDuration;
    /**
     * 柱塞延迟时间
     */
    private int plungerDelayDuration;

    public SimpleLogicMode() {
        this.setCode("SCSS.SIMPLE_LOGIC_MODE");
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(130);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0096    0020     40
        command.append("01100096002040");
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenTimeTimestamp), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenPressureValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenOilPressureValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenLoadFactorPresets), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseTimeTimestamp), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellClosePressureValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseOilPressureValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseFlowValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inWellOpenTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.axWellOpenTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inWellCloseTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.axWellCloseTimeDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.ressureStabilizationDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.lowStabilizationDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.loadFactorStabilizationDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.plungerDelayDuration), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }
}
